@@ -18,16 +18,22 @@
 
 from __future__ import annotations
 
+import asyncio
 import typing as t
+import warnings
 
 
 if t.TYPE_CHECKING:
+    import ssl
     import typing_extensions as te
 
-    import ssl
-
+    from .._api import (
+        T_NotificationDisabledCategory,
+        T_NotificationMinimumSeverity,
+    )
 
 
+from .._api import RoutingControl
 from .._async_compat.util import AsyncUtil
 from .._conf import (
     Config,
@@ -41,8 +47,10 @@ from .._meta import (
     deprecation_warn,
     experimental,
     experimental_warn,
+    ExperimentalWarning,
     unclosed_resource_warn,
 )
+from .._work import EagerResult
 from ..addressing import Address
 from ..api import (
     AsyncBookmarkManager,
@@ -71,7 +79,31 @@ from .bookmark_manager import (
     TBmConsumer as _TBmConsumer,
     TBmSupplier as _TBmSupplier,
 )
-from .work import AsyncSession
+from .work import (
+    AsyncManagedTransaction,
+    AsyncResult,
+    AsyncSession,
+)
+
+
+if t.TYPE_CHECKING:
+    import ssl
+    from enum import Enum
+
+    import typing_extensions as te
+
+    from .._api import T_RoutingControl
+
+
+    class _DefaultEnum(Enum):
+        default = "default"
+
+    _default = _DefaultEnum.default
+
+else:
+    _default = object()
+
+_T = t.TypeVar("_T")
 
 
 class AsyncGraphDatabase:
@@ -102,6 +134,12 @@ class AsyncGraphDatabase:
             ssl_context: ssl.SSLContext = ...,
             user_agent: str = ...,
             keep_alive: bool = ...,
+            notifications_min_severity: t.Optional[
+                T_NotificationMinimumSeverity
+            ] = ...,
+            notifications_disabled_categories: t.Optional[
+                t.Iterable[T_NotificationDisabledCategory]
+            ] = ...,
 
             # undocumented/unsupported options
             # they may be change or removed any time without prior notice
@@ -121,17 +159,25 @@ class AsyncGraphDatabase:
     else:
 
         @classmethod
-        def driver(cls, uri, *, auth=None, **config) -> AsyncDriver:
+        def driver(
+            cls, uri: str, *,
+            auth: t.Union[t.Tuple[t.Any, t.Any], Auth, None] = None,
+            **config
+        ) -> AsyncDriver:
             """Create a driver.
 
-            :param uri: the connection URI for the driver, see :ref:`async-uri-ref` for available URIs.
-            :param auth: the authentication details, see :ref:`auth-ref` for available authentication details.
-            :param config: driver configuration key-word arguments, see :ref:`async-driver-configuration-ref` for available key-word arguments.
+            :param uri: the connection URI for the driver,
+                see :ref:`async-uri-ref` for available URIs.
+            :param auth: the authentication details,
+                see :ref:`auth-ref` for available authentication details.
+            :param config: driver configuration key-word arguments,
+                see :ref:`async-driver-configuration-ref` for available
+                key-word arguments.
             """
 
             driver_type, security_type, parsed = parse_neo4j_uri(uri)
 
-            # TODO: 6.0 remove "trust" config option
+            # TODO: 6.0 - remove "trust" config option
             if "trust" in config.keys():
                 if config["trust"] not in (
                     TRUST_ALL_CERTIFICATES,
@@ -166,7 +212,7 @@ class AsyncGraphDatabase:
                      or "ssl_context" in config.keys())):
                 from ..exceptions import ConfigurationError
 
-                # TODO: 6.0 remove "trust" from error message
+                # TODO: 6.0 - remove "trust" from error message
                 raise ConfigurationError(
                     'The config settings "encrypted", "trust", '
                     '"trusted_certificates", and "ssl_context" can only be '
@@ -191,6 +237,7 @@ class AsyncGraphDatabase:
             elif security_type == SECURITY_TYPE_SELF_SIGNED_CERTIFICATE:
                 config["encrypted"] = True
                 config["trusted_certificates"] = TrustAll()
+            _normalize_notifications_config(config)
 
             assert driver_type in (DRIVER_BOLT, DRIVER_NEO4J)
             if driver_type == DRIVER_BOLT:
@@ -274,7 +321,7 @@ class AsyncGraphDatabase:
             Function which will be called whenever the set of bookmarks
             handled by the bookmark manager gets updated with the new
             internal bookmark set. It will receive the new set of bookmarks
-            as a :class:`.Bookmarks` object and return :const:`None`.
+            as a :class:`.Bookmarks` object and return :data:`None`.
 
         :returns: A default implementation of :class:`AsyncBookmarkManager`.
 
@@ -401,6 +448,12 @@ class AsyncDriver:
         assert default_workspace_config is not None
         self._pool = pool
         self._default_workspace_config = default_workspace_config
+        with warnings.catch_warnings():
+            warnings.filterwarnings("ignore",
+                                    message=r".*\bbookmark manager\b.*",
+                                    category=ExperimentalWarning)
+            self._query_bookmark_manager = \
+                AsyncGraphDatabase.bookmark_manager()
 
     async def __aenter__(self) -> AsyncDriver:
         return self
@@ -428,10 +481,15 @@ class AsyncDriver:
         """Indicate whether the driver was configured to use encryption."""
         return bool(self._pool.pool_config.encrypted)
 
+    def _prepare_session_config(self, **config):
+        _normalize_notifications_config(config)
+        return config
+
     if t.TYPE_CHECKING:
 
         def session(
             self,
+            *,
             connection_acquisition_timeout: float = ...,
             max_transaction_retry_time: float = ...,
             database: t.Optional[str] = ...,
@@ -441,6 +499,12 @@ class AsyncDriver:
             default_access_mode: str = ...,
             bookmark_manager: t.Union[AsyncBookmarkManager,
                                       BookmarkManager, None] = ...,
+            notifications_min_severity: t.Optional[
+                T_NotificationMinimumSeverity
+            ] = ...,
+            notifications_disabled_categories: t.Optional[
+                t.Iterable[T_NotificationDisabledCategory]
+            ] = ...,
 
             # undocumented/unsupported options
             # they may be change or removed any time without prior notice
@@ -466,13 +530,334 @@ class AsyncDriver:
     async def close(self) -> None:
         """ Shut down, closing any open connections in the pool.
         """
-        await self._pool.close()
+        try:
+            await self._pool.close()
+        except asyncio.CancelledError:
+            self._closed = True
+            raise
         self._closed = True
 
+    # overloads to work around https://github.com/python/mypy/issues/3737
+    @t.overload
+    async def execute_query(
+        self,
+        query_: str,
+        parameters_: t.Optional[t.Dict[str, t.Any]] = None,
+        routing_: T_RoutingControl = RoutingControl.WRITERS,
+        database_: t.Optional[str] = None,
+        impersonated_user_: t.Optional[str] = None,
+        bookmark_manager_: t.Union[
+            AsyncBookmarkManager, BookmarkManager, None
+        ] = ...,
+        result_transformer_: t.Callable[
+            [AsyncResult], t.Awaitable[EagerResult]
+        ] = ...,
+        **kwargs: t.Any
+    ) -> EagerResult:
+        ...
+
+    @t.overload
+    async def execute_query(
+        self,
+        query_: str,
+        parameters_: t.Optional[t.Dict[str, t.Any]] = None,
+        routing_: T_RoutingControl = RoutingControl.WRITERS,
+        database_: t.Optional[str] = None,
+        impersonated_user_: t.Optional[str] = None,
+        bookmark_manager_: t.Union[
+            AsyncBookmarkManager, BookmarkManager, None
+        ] = ...,
+        result_transformer_: t.Callable[
+            [AsyncResult], t.Awaitable[_T]
+        ] = ...,
+        **kwargs: t.Any
+    ) -> _T:
+        ...
+
+    @experimental(
+        "Driver.execute_query is experimental. "
+        "It might be changed or removed any time even without prior notice."
+    )
+    async def execute_query(
+        self,
+        query_: str,
+        parameters_: t.Optional[t.Dict[str, t.Any]] = None,
+        routing_: T_RoutingControl = RoutingControl.WRITERS,
+        database_: t.Optional[str] = None,
+        impersonated_user_: t.Optional[str] = None,
+        bookmark_manager_: t.Union[
+            AsyncBookmarkManager, BookmarkManager, None,
+            te.Literal[_DefaultEnum.default]
+        ] = _default,
+        result_transformer_: t.Callable[
+            [AsyncResult], t.Awaitable[t.Any]
+        ] = AsyncResult.to_eager_result,
+        **kwargs: t.Any
+    ) -> t.Any:
+        """Execute a query in a transaction function and return all results.
+
+        This method is a handy wrapper for lower-level driver APIs like
+        sessions, transactions, and transaction functions. It is intended
+        for simple use cases where there is no need for managing all possible
+        options.
+
+        The internal usage of transaction functions provides a retry-mechanism
+        for appropriate errors. Furthermore, this means that queries using
+        ``CALL {} IN TRANSACTIONS`` or the older ``USING PERIODIC COMMIT``
+        will not work (use :meth:`AsyncSession.run` for these).
+
+        The method is roughly equivalent to::
+
+            async def execute_query(
+                query_, parameters_, routing_, database_, impersonated_user_,
+                bookmark_manager_, result_transformer_, **kwargs
+            ):
+                async def work(tx):
+                    result = await tx.run(query_, parameters_, **kwargs)
+                    return await result_transformer_(result)
+
+                async with driver.session(
+                    database=database_,
+                    impersonated_user=impersonated_user_,
+                    bookmark_manager=bookmark_manager_,
+                ) as session:
+                    if routing_ == RoutingControl.WRITERS:
+                        return await session.execute_write(work)
+                    elif routing_ == RoutingControl.READERS:
+                        return await session.execute_read(work)
+
+        Usage example::
+
+            from typing import List
+
+            import neo4j
+
+
+            async def example(driver: neo4j.AsyncDriver) -> List[str]:
+                \"""Get the name of all 42 year-olds.\"""
+                records, summary, keys = await driver.execute_query(
+                    "MATCH (p:Person {age: $age}) RETURN p.name",
+                    {"age": 42},
+                    routing_=neo4j.RoutingControl.READERS,  # or just "r"
+                    database_="neo4j",
+                )
+                assert keys == ["p.name"]  # not needed, just for illustration
+                # log_summary(summary)  # log some metadata
+                return [str(record["p.name"]) for record in records]
+                # or: return [str(record[0]) for record in records]
+                # or even: return list(map(lambda r: str(r[0]), records))
+
+        Another example::
+
+            import neo4j
+
+
+            async def example(driver: neo4j.AsyncDriver) -> int:
+                \"""Call all young people "My dear" and get their count.\"""
+                record = await driver.execute_query(
+                    "MATCH (p:Person) WHERE p.age <= 15 "
+                    "SET p.nickname = 'My dear' "
+                    "RETURN count(*)",
+                    # optional routing parameter, as write is default
+                    # routing_=neo4j.RoutingControl.WRITERS,  # or just "w",
+                    database_="neo4j",
+                    result_transformer_=neo4j.AsyncResult.single,
+                )
+                assert record is not None  # for typechecking and illustration
+                count = record[0]
+                assert isinstance(count, int)
+                return count
+
+        :param query_: cypher query to execute
+        :type query_: typing.Optional[str]
+        :param parameters_: parameters to use in the query
+        :type parameters_: typing.Optional[typing.Dict[str, typing.Any]]
+        :param routing_:
+            whether to route the query to a reader (follower/read replica) or
+            a writer (leader) in the cluster. Default is to route to a writer.
+        :type routing_: neo4j.RoutingControl
+        :param database_:
+            database to execute the query against.
+
+            None (default) uses the database configured on the server side.
+
+            .. Note::
+                It is recommended to always specify the database explicitly
+                when possible. This allows the driver to work more efficiently,
+                as it will not have to resolve the default database first.
+
+            See also the Session config :ref:`database-ref`.
+        :type database_: typing.Optional[str]
+        :param impersonated_user_:
+            Name of the user to impersonate.
+
+            This means that all query will be executed in the security context
+            of the impersonated user. For this, the user for which the
+            :class:`Driver` has been created needs to have the appropriate
+            permissions.
+
+            See also the Session config :ref:`impersonated-user-ref`.
+        :type impersonated_user_: typing.Optional[str]
+        :param result_transformer_:
+            A function that gets passed the :class:`neo4j.AsyncResult` object
+            resulting from the query and converts it to a different type. The
+            result of the transformer function is returned by this method.
+
+            .. warning::
+
+                The transformer function must **not** return the
+                :class:`neo4j.AsyncResult` itself.
+
+            .. warning::
+
+                N.B. the driver might retry the underlying transaction so the
+                transformer might get invoked more than once (with different
+                :class:`neo4j.AsyncResult` objects).
+                Therefore, it needs to be idempotent (i.e., have the same
+                effect, regardless if called once or many times).
+
+            Example transformer that checks that exactly one record is in the
+            result stream, then returns the record and the result summary::
+
+                from typing import Tuple
+
+                import neo4j
+
+
+                async def transformer(
+                    result: neo4j.AsyncResult
+                ) -> Tuple[neo4j.Record, neo4j.ResultSummary]:
+                    record = await result.single(strict=True)
+                    summary = await result.consume()
+                    return record, summary
+
+            Note that methods of :class:`neo4j.AsyncResult` that don't take
+            mandatory arguments can be used directly as transformer functions.
+            For example::
+
+                import neo4j
+
+
+                async def example(driver: neo4j.AsyncDriver) -> neo4j.Record::
+                    record = await driver.execute_query(
+                        "SOME QUERY",
+                        result_transformer_=neo4j.AsyncResult.single
+                    )
+
+
+                # is equivalent to:
+
+
+                async def transformer(result: neo4j.AsyncResult) -> neo4j.Record:
+                    return await result.single()
+
+
+                async def example(driver: neo4j.AsyncDriver) -> neo4j.Record::
+                    record = await driver.execute_query(
+                        "SOME QUERY",
+                        result_transformer_=transformer
+                    )
+
+        :type result_transformer_:
+            typing.Callable[[neo4j.AsyncResult], typing.Awaitable[T]]
+        :param bookmark_manager_:
+            Specify a bookmark manager to use.
+
+            If present, the bookmark manager is used to keep the query causally
+            consistent with all work executed using the same bookmark manager.
+
+            Defaults to the driver's :attr:`.query_bookmark_manager`.
+
+            Pass :const:`None` to disable causal consistency.
+        :type bookmark_manager_:
+            typing.Union[neo4j.AsyncBookmarkManager, neo4j.BookmarkManager,
+                         None]
+        :param kwargs: additional keyword parameters. None of these can end
+            with a single underscore. This is to avoid collisions with the
+            keyword configuration parameters of this method. If you need to
+            pass such a parameter, use the ``parameters_`` parameter instead.
+            These take precedence over parameters passed as ``parameters_``.
+        :type kwargs: typing.Any
+
+        :returns: the result of the ``result_transformer``
+        :rtype: T
+
+        **This is experimental.** (See :ref:`filter-warnings-ref`)
+        It might be changed or removed any time even without prior notice.
+
+        .. versionadded:: 5.5
+        """
+        invalid_kwargs = [k for k in kwargs if
+                          k[-2:-1] != "_" and k[-1:] == "_"]
+        if invalid_kwargs:
+            raise ValueError(
+                "keyword parameters must not end with a single '_'. Found: %r"
+                "\nYou either misspelled an existing configuration parameter "
+                "or tried to send a query parameter that is reserved. In the "
+                "latter case, use the `parameters_` dictionary instead."
+                % invalid_kwargs
+            )
+        parameters = dict(parameters_ or {}, **kwargs)
+
+        if bookmark_manager_ is _default:
+            bookmark_manager_ = self._query_bookmark_manager
+        assert bookmark_manager_ is not _default
+
+        with warnings.catch_warnings():
+            warnings.filterwarnings("ignore",
+                                    message=r".*\bbookmark_manager\b.*",
+                                    category=ExperimentalWarning)
+            session = self.session(database=database_,
+                                   impersonated_user=impersonated_user_,
+                                   bookmark_manager=bookmark_manager_)
+        async with session:
+            if routing_ == RoutingControl.WRITERS:
+                executor = session.execute_write
+            elif routing_ == RoutingControl.READERS:
+                executor = session.execute_read
+            else:
+                raise ValueError("Invalid routing control value: %r"
+                                 % routing_)
+            return await executor(
+                _work, query_, parameters, result_transformer_
+            )
+
+    @property
+    @experimental(
+        "Driver.query_bookmark_manager is experimental. "
+        "It might be changed or removed any time even without prior notice."
+    )
+    def query_bookmark_manager(self) -> AsyncBookmarkManager:
+        """The driver's default query bookmark manager.
+
+        This is the default :class:`AsyncBookmarkManager` used by
+        :meth:`.execute_query`. This can be used to causally chain
+        :meth:`.execute_query` calls and sessions. Example::
+
+            async def example(driver: neo4j.AsyncDriver) -> None:
+                await driver.execute_query("<QUERY 1>")
+                async with driver.session(
+                    bookmark_manager=driver.query_bookmark_manager
+                ) as session:
+                    # every query inside this session will be causally chained
+                    # (i.e., can read what was written by <QUERY 1>)
+                    await session.run("<QUERY 2>")
+                # subsequent execute_query calls will be causally chained
+                # (i.e., can read what was written by <QUERY 2>)
+                await driver.execute_query("<QUERY 3>")
+
+        **This is experimental.** (See :ref:`filter-warnings-ref`)
+        It might be changed or removed any time even without prior notice.
+
+        .. versionadded:: 5.5
+        """
+        return self._query_bookmark_manager
+
     if t.TYPE_CHECKING:
 
         async def verify_connectivity(
             self,
+            *,
             # all arguments are experimental
             # they may be change or removed any time without prior notice
             session_connection_timeout: float = ...,
@@ -485,6 +870,12 @@ class AsyncDriver:
             default_access_mode: str = ...,
             bookmark_manager: t.Union[AsyncBookmarkManager,
                                       BookmarkManager, None] = ...,
+            notifications_min_severity: t.Optional[
+                T_NotificationMinimumSeverity
+            ] = ...,
+            notifications_disabled_categories: t.Optional[
+                t.Iterable[T_NotificationDisabledCategory]
+            ] = ...,
 
             # undocumented/unsupported options
             initial_retry_delay: float = ...,
@@ -537,6 +928,7 @@ class AsyncDriver:
 
         async def get_server_info(
             self,
+            *,
             # all arguments are experimental
             # they may be change or removed any time without prior notice
             session_connection_timeout: float = ...,
@@ -549,6 +941,12 @@ class AsyncDriver:
             default_access_mode: str = ...,
             bookmark_manager: t.Union[AsyncBookmarkManager,
                                       BookmarkManager, None] = ...,
+            notifications_min_severity: t.Optional[
+                T_NotificationMinimumSeverity
+            ] = ...,
+            notifications_disabled_categories: t.Optional[
+                t.Iterable[T_NotificationDisabledCategory]
+            ] = ...,
 
             # undocumented/unsupported options
             initial_retry_delay: float = ...,
@@ -614,6 +1012,22 @@ class AsyncDriver:
             return session._connection.supports_multiple_databases
 
 
+async def _work(
+    tx: AsyncManagedTransaction,
+    query: str,
+    parameters: t.Dict[str, t.Any],
+    transformer: t.Callable[[AsyncResult], t.Awaitable[_T]]
+) -> _T:
+    res = await tx.run(query, parameters)
+    if transformer is AsyncResult.to_eager_result:
+        with warnings.catch_warnings():
+            warnings.filterwarnings("ignore",
+                                    message=r".*\bto_eager_result\b.*",
+                                    category=ExperimentalWarning)
+            return await transformer(res)
+    return await transformer(res)
+
+
 class AsyncBoltDriver(_Direct, AsyncDriver):
     """:class:`.AsyncBoltDriver` is instantiated for ``bolt`` URIs and
     addresses a single database machine. This may be a standalone server or
@@ -657,6 +1071,7 @@ class AsyncBoltDriver(_Direct, AsyncDriver):
             :returns:
             :rtype: :class: `neo4j.AsyncSession`
             """
+            config = self._prepare_session_config(**config)
             session_config = SessionConfig(self._default_workspace_config,
                                            config)
             return AsyncSession(self._pool, session_config)
@@ -688,6 +1103,20 @@ class AsyncNeo4jDriver(_Routing, AsyncDriver):
     if not t.TYPE_CHECKING:
 
         def session(self, **config) -> AsyncSession:
+            config = self._prepare_session_config(**config)
             session_config = SessionConfig(self._default_workspace_config,
                                            config)
             return AsyncSession(self._pool, session_config)
+
+
+def _normalize_notifications_config(config):
+    if config.get("notifications_disabled_categories") is not None:
+        config["notifications_disabled_categories"] = [
+            getattr(e, "value", e)
+            for e in config["notifications_disabled_categories"]
+        ]
+    if config.get("notifications_min_severity") is not None:
+        config["notifications_min_severity"] = getattr(
+            config["notifications_min_severity"], "value",
+            config["notifications_min_severity"]
+        )
