@@ -19,15 +19,16 @@
 from __future__ import annotations
 
 import typing as t
-from collections import deque
+from collections import (
+    deque,
+    namedtuple,
+)
 from warnings import warn
 
 
 if t.TYPE_CHECKING:
     import typing_extensions as te
 
-
-
 from ..._async_compat.util import AsyncUtil
 from ..._codec.hydration import BrokenHydrationObject
 from ..._data import (
@@ -35,6 +36,10 @@ from ..._data import (
     RecordTableRowExporter,
 )
 from ..._meta import experimental
+from ..._work import (
+    EagerResult,
+    ResultSummary,
+)
 from ...exceptions import (
     ResultConsumedError,
     ResultNotSingleError,
@@ -43,7 +48,6 @@ from ...time import (
     Date,
     DateTime,
 )
-from ...work import ResultSummary
 from ..io import ConnectionErrorHandler
 
 
@@ -113,10 +117,11 @@ class AsyncResult:
         # BEGIN+RUN does not carry any extra on the RUN message.
         # BEGIN {extra}
         # RUN "query" {parameters} {extra}
-        await self._run(query, parameters, None, None, None, None)
+        await self._run(query, parameters, None, None, None, None, None, None)
 
     async def _run(
-        self, query, parameters, db, imp_user, access_mode, bookmarks
+        self, query, parameters, db, imp_user, access_mode, bookmarks,
+        notifications_min_severity, notifications_disabled_categories
     ):
         query_text = str(query)  # Query or string object
         query_metadata = getattr(query, "metadata", None)
@@ -153,6 +158,9 @@ class AsyncResult:
             timeout=query_timeout,
             db=db,
             imp_user=imp_user,
+            notifications_min_severity=notifications_min_severity,
+            notifications_disabled_categories=
+                notifications_disabled_categories,
             dehydration_hooks=self._hydration_scope.dehydration_hooks,
             on_success=on_attached,
             on_failure=on_failed_attach,
@@ -418,7 +426,7 @@ class AsyncResult:
         there is not exactly one record left.
 
         If ``strict`` is :const:`False`, fewer than one record will make this
-        method return :const:`None`, more than one record will make this method
+        method return :data:`None`, more than one record will make this method
         emit a warning and return the first record.
 
         :param strict:
@@ -427,7 +435,7 @@ class AsyncResult:
             warning if there are more than 1 record.
             :const:`False` by default.
 
-        :returns: the next :class:`neo4j.Record` or :const:`None` if none remain
+        :returns: the next :class:`neo4j.Record` or :data:`None` if none remain
 
         :warns: if more than one record is available and
             ``strict`` is :const:`False`
@@ -496,7 +504,7 @@ class AsyncResult:
 
         This leaves the record in the buffer for further processing.
 
-        :returns: the next :class:`neo4j.Record` or :const:`None` if none
+        :returns: the next :class:`neo4j.Record` or :data:`None` if none
             remain.
 
         :raises ResultConsumedError: if the transaction from which this result
@@ -600,6 +608,35 @@ class AsyncResult:
         """
         return [record.data(*keys) async for record in self]
 
+    @experimental(
+        "Result.to_eager_result is experimental. "
+        "It might be changed or removed any time even without prior notice."
+    )
+    async def to_eager_result(self) -> EagerResult:
+        """Convert this result to an :class:`.EagerResult`.
+
+        This method exhausts the result and triggers a :meth:`.consume`.
+
+        :returns: all remaining records in the result stream, the result's
+            summary, and keys as an :class:`.EagerResult` instance.
+
+        :raises ResultConsumedError: if the transaction from which this result
+            was obtained has been closed or the Result has been explicitly
+            consumed.
+
+        **This is experimental.** (See :ref:`filter-warnings-ref`)
+        It might be changed or removed any time even without prior notice.
+
+        .. versionadded:: 5.5
+        """
+
+        await self._buffer_all()
+        return EagerResult(
+            keys=list(self.keys()),
+            records=await AsyncUtil.list(self),
+            summary=await self.consume()
+        )
+
     async def to_df(
         self,
         expand: bool = False,
@@ -611,7 +648,7 @@ class AsyncResult:
 
         ::
 
-            res = await tx.run("UNWIND range(1, 10) AS n RETURN n, n+1 as m")
+            res = await tx.run("UNWIND range(1, 10) AS n RETURN n, n+1 AS m")
             df = await res.to_df()
 
         for instance will return a DataFrame with two columns: ``n`` and ``m``
@@ -683,7 +720,7 @@ class AsyncResult:
         :param parse_dates:
             If :const:`True`, columns that exclusively contain
             :class:`time.DateTime` objects, :class:`time.Date` objects, or
-            :const:`None`, will be converted to :class:`pandas.Timestamp`.
+            :data:`None`, will be converted to :class:`pandas.Timestamp`.
 
         :raises ImportError: if `pandas` library is not available.
         :raises ResultConsumedError: if the transaction from which this result
@@ -730,9 +767,9 @@ class AsyncResult:
         df[dt_columns] = df[dt_columns].apply(
             lambda col: col.map(
                 lambda x:
-                    pd.Timestamp(x.iso_format())
-                        .replace(tzinfo=getattr(x, "tzinfo", None))
-                    if x else pd.NaT
+                pd.Timestamp(x.iso_format())
+                .replace(tzinfo=getattr(x, "tzinfo", None))
+                if x else pd.NaT
             )
         )
         return df
