@@ -62,6 +62,8 @@ class Bolt4x0(Bolt):
 
     supports_multiple_databases = True
 
+    supports_notification_filtering = False
+
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self._server_state_manager = ServerStateManager(
@@ -96,6 +98,11 @@ class Bolt4x0(Bolt):
         }
 
     def hello(self, dehydration_hooks=None, hydration_hooks=None):
+        if (
+            self.notifications_min_severity is not None
+            or self.notifications_disabled_categories is not None
+        ):
+            self.assert_notification_filtering_support()
         headers = self.get_base_headers()
         headers.update(self.auth_dict)
         logged_headers = dict(headers)
@@ -157,7 +164,9 @@ class Bolt4x0(Bolt):
 
     def run(self, query, parameters=None, mode=None, bookmarks=None,
             metadata=None, timeout=None, db=None, imp_user=None,
-            dehydration_hooks=None, hydration_hooks=None, **handlers):
+            notifications_min_severity=None,
+            notifications_disabled_categories=None, dehydration_hooks=None,
+            hydration_hooks=None, **handlers):
         if imp_user is not None:
             raise ConfigurationError(
                 "Impersonation is not supported in Bolt Protocol {!r}. "
@@ -165,6 +174,11 @@ class Bolt4x0(Bolt):
                     self.PROTOCOL_VERSION, imp_user
                 )
             )
+        if (
+            notifications_min_severity is not None
+            or notifications_disabled_categories is not None
+        ):
+            self.assert_notification_filtering_support()
         if not parameters:
             parameters = {}
         extra = {}
@@ -216,7 +230,8 @@ class Bolt4x0(Bolt):
                      dehydration_hooks=dehydration_hooks)
 
     def begin(self, mode=None, bookmarks=None, metadata=None, timeout=None,
-              db=None, imp_user=None, dehydration_hooks=None,
+              db=None, imp_user=None, notifications_min_severity=None,
+              notifications_disabled_categories=None, dehydration_hooks=None,
               hydration_hooks=None, **handlers):
         if imp_user is not None:
             raise ConfigurationError(
@@ -225,6 +240,11 @@ class Bolt4x0(Bolt):
                     self.PROTOCOL_VERSION, imp_user
                 )
             )
+        if (
+            notifications_min_severity is not None
+            or notifications_disabled_categories is not None
+        ):
+            self.assert_notification_filtering_support()
         extra = {}
         if mode in (READ_ACCESS, "r"):
             extra["mode"] = "r"  # It will default to mode "w" if nothing is specified
@@ -333,7 +353,7 @@ class Bolt4x0(Bolt):
                     self.pool.on_write_failure(address=self.unresolved_address)
                 raise
             except Neo4jError as e:
-                if self.pool and e.invalidates_all_connections():
+                if self.pool and e._invalidates_all_connections():
                     self.pool.mark_all_stale()
                 raise
         else:
@@ -416,6 +436,12 @@ class Bolt4x3(Bolt4x2):
         return [metadata.get("rt")]
 
     def hello(self, dehydration_hooks=None, hydration_hooks=None):
+        if (
+            self.notifications_min_severity is not None
+            or self.notifications_disabled_categories is not None
+        ):
+            self.assert_notification_filtering_support()
+
         def on_success(metadata):
             self.configuration_hints.update(metadata.pop("hints", {}))
             self.server_info.update(metadata)
@@ -485,7 +511,14 @@ class Bolt4x4(Bolt4x3):
 
     def run(self, query, parameters=None, mode=None, bookmarks=None,
             metadata=None, timeout=None, db=None, imp_user=None,
-            dehydration_hooks=None, hydration_hooks=None, **handlers):
+            notifications_min_severity=None,
+            notifications_disabled_categories=None, dehydration_hooks=None,
+            hydration_hooks=None, **handlers):
+        if (
+            notifications_min_severity is not None
+            or notifications_disabled_categories is not None
+        ):
+            self.assert_notification_filtering_support()
         if not parameters:
             parameters = {}
         extra = {}
@@ -521,8 +554,14 @@ class Bolt4x4(Bolt4x3):
                      dehydration_hooks=dehydration_hooks)
 
     def begin(self, mode=None, bookmarks=None, metadata=None, timeout=None,
-              db=None, imp_user=None, dehydration_hooks=None,
+              db=None, imp_user=None, notifications_min_severity=None,
+              notifications_disabled_categories=None, dehydration_hooks=None,
               hydration_hooks=None, **handlers):
+        if (
+            notifications_min_severity is not None
+            or notifications_disabled_categories is not None
+        ):
+            self.assert_notification_filtering_support()
         extra = {}
         if mode in (READ_ACCESS, "r"):
             # It will default to mode "w" if nothing is specified
