@@ -19,15 +19,16 @@
 from __future__ import annotations
 
 import typing as t
-from collections import deque
+from collections import (
+    deque,
+    namedtuple,
+)
 from warnings import warn
 
 
 if t.TYPE_CHECKING:
     import typing_extensions as te
 
-
-
 from ..._async_compat.util import Util
 from ..._codec.hydration import BrokenHydrationObject
 from ..._data import (
@@ -35,6 +36,10 @@ from ..._data import (
     RecordTableRowExporter,
 )
 from ..._meta import experimental
+from ..._work import (
+    EagerResult,
+    ResultSummary,
+)
 from ...exceptions import (
     ResultConsumedError,
     ResultNotSingleError,
@@ -43,7 +48,6 @@ from ...time import (
     Date,
     DateTime,
 )
-from ...work import ResultSummary
 from ..io import ConnectionErrorHandler
 
 
@@ -113,10 +117,11 @@ class Result:
         # BEGIN+RUN does not carry any extra on the RUN message.
         # BEGIN {extra}
         # RUN "query" {parameters} {extra}
-        self._run(query, parameters, None, None, None, None)
+        self._run(query, parameters, None, None, None, None, None, None)
 
     def _run(
-        self, query, parameters, db, imp_user, access_mode, bookmarks
+        self, query, parameters, db, imp_user, access_mode, bookmarks,
+        notifications_min_severity, notifications_disabled_categories
     ):
         query_text = str(query)  # Query or string object
         query_metadata = getattr(query, "metadata", None)
@@ -153,6 +158,9 @@ class Result:
             timeout=query_timeout,
             db=db,
             imp_user=imp_user,
+            notifications_min_severity=notifications_min_severity,
+            notifications_disabled_categories=
+                notifications_disabled_categories,
             dehydration_hooks=self._hydration_scope.dehydration_hooks,
             on_success=on_attached,
             on_failure=on_failed_attach,
@@ -600,6 +608,35 @@ class Result:
         """
         return [record.data(*keys) for record in self]
 
+    @experimental(
+        "Result.to_eager_result is experimental. "
+        "It might be changed or removed any time even without prior notice."
+    )
+    def to_eager_result(self) -> EagerResult:
+        """Convert this result to an :class:`.EagerResult`.
+
+        This method exhausts the result and triggers a :meth:`.consume`.
+
+        :returns: all remaining records in the result stream, the result's
+            summary, and keys as an :class:`.EagerResult` instance.
+
+        :raises ResultConsumedError: if the transaction from which this result
+            was obtained has been closed or the Result has been explicitly
+            consumed.
+
+        **This is experimental.** (See :ref:`filter-warnings-ref`)
+        It might be changed or removed any time even without prior notice.
+
+        .. versionadded:: 5.5
+        """
+
+        self._buffer_all()
+        return EagerResult(
+            keys=list(self.keys()),
+            records=Util.list(self),
+            summary=self.consume()
+        )
+
     def to_df(
         self,
         expand: bool = False,
@@ -730,9 +767,9 @@ class Result:
         df[dt_columns] = df[dt_columns].apply(
             lambda col: col.map(
                 lambda x:
-                    pd.Timestamp(x.iso_format())
-                        .replace(tzinfo=getattr(x, "tzinfo", None))
-                    if x else pd.NaT
+                pd.Timestamp(x.iso_format())
+                .replace(tzinfo=getattr(x, "tzinfo", None))
+                if x else pd.NaT
             )
         )
         return df
